from django.core.management.base import BaseCommand

from wagtail_localize.models import LocaleSynchronization
from wagtail_localize.synctree import PageIndex


class Command(BaseCommand):
    help = "Synchronises the structure of all locale page trees so they contain the same pages. Creates alias pages where necessary."

    def handle(self, **options):
        page_index = PageIndex.from_database().sort_by_tree_position()
        for locale_sync in LocaleSynchronization.objects.all():
            locale_sync.sync_trees(page_index=page_index)



from django.conf import settings
from django.utils.module_loading import import_string


def get_machine_translator():
    config = getattr(settings, "WAGTAILLOCALIZE_MACHINE_TRANSLATOR", None)

    if config is None:
        return

    # Raises ImportError
    machine_translator_class = import_string(config["CLASS"])

    return machine_translator_class(config.get("OPTIONS", {}))



from django.contrib.auth.models import Permission
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from wagtail.admin.menu import MenuItem
from wagtail.core import hooks
from wagtail.core.permissions import site_permission_policy

from .views import LocaleViewSet


@hooks.register("register_admin_viewset")
def register_viewset():
    return LocaleViewSet("wagtaillocales", url_prefix="locales")


class LocalesMenuItem(MenuItem):
    def is_shown(self, request):
        return site_permission_policy.user_has_any_permission(
            request.user, ["add", "change", "delete"]
        )


@hooks.register("register_settings_menu_item")
def register_locales_menu_item():
    return LocalesMenuItem(
        _("Locales"), reverse("wagtaillocales:index"), icon_name="site", order=603
    )


@hooks.register("register_permissions")
def register_permissions():
    return Permission.objects.filter(
        content_type__app_label="wagtailcore",
        codename__in=["add_locale", "change_locale", "delete_locale"],
    )



from django.apps import AppConfig


class WagtailLocalizeAppConfig(AppConfig):
    name = "wagtail_localize"
    label = "wagtail_localize"
    verbose_name = "Wagtail localize"
    default_auto_field = "django.db.models.AutoField"

    def ready(self):
        from .models import register_post_delete_signal_handlers

        register_post_delete_signal_handlers()



# Generated by Django 3.0.6 on 2020-07-20 17:27

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0057_page_locale_fields_notnull"),
        ("wagtail_localize", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Translation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("uuid", models.UUIDField(default=uuid.uuid4, unique=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("source_last_updated_at", models.DateTimeField(auto_now_add=True)),
                ("translations_last_updated_at", models.DateTimeField(null=True)),
                ("destination_last_updated_at", models.DateTimeField(null=True)),
                ("enabled", models.BooleanField(default=True)),
                (
                    "object",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="wagtail_localize.TranslatableObject",
                    ),
                ),
                (
                    "source",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="wagtail_localize.TranslationSource",
                    ),
                ),
                (
                    "target_locale",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="wagtailcore.Locale",
                    ),
                ),
            ],
            options={
                "unique_together": {("object", "target_locale")},
            },
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-23 10:24

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("wagtailcore", "0059_apply_collection_ordering"),
        ("wagtail_localize", "0010_overridablesegment"),
    ]

    operations = [
        migrations.CreateModel(
            name="SegmentOverride",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("data_json", models.TextField()),
                ("has_error", models.BooleanField(default=False)),
                ("field_error", models.TextField(blank=True)),
                (
                    "context",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="overrides",
                        to="wagtail_localize.translationcontext",
                    ),
                ),
                (
                    "last_translated_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "locale",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="overrides",
                        to="wagtailcore.locale",
                    ),
                ),
            ],
        ),
    ]



from django.urls import reverse
from wagtail.admin.ui.side_panels import PagePreviewSidePanel, PageSidePanels
from wagtail.models import PreviewableMixin


class LocalizedPreviewSidePanel(PagePreviewSidePanel):
    def __init__(self, object, request, translation):
        super().__init__(object, request)
        self.translation = translation

    def get_context_data(self, parent_context):
        context = super().get_context_data(parent_context)

        if isinstance(object, PreviewableMixin) and object.is_previewable():
            preview_modes = [
                {
                    "mode": mode,
                    "label": label,
                    "url": reverse(
                        "wagtail_localize:preview_translation",
                        args=[self.translation.id],
                    )
                    if mode == self.object.default_preview_mode
                    else reverse(
                        "wagtail_localize:preview_translation",
                        args=[self.translation.id, mode],
                    ),
                }
                for mode, label in self.object.preview_modes
            ]
        else:
            preview_modes = []

        for mode in preview_modes:
            context["preview_url"] = mode["url"]

        return context


class LocalizedPageSidePanels(PageSidePanels):
    def __init__(self, request, page, translation):
        super().__init__(request, page, preview_enabled=False, comments_enabled=False)

        # FIXME: enable the preview panels, with an updated JS handler, as preview-panels.js expects a regular form
        # self.side_panels += [
        #     LocalizedPreviewSidePanel(page, self.request, translation),
        # ]



from google.cloud import translate

from wagtail_localize.strings import StringValue

from .base import BaseMachineTranslator


class GoogleCloudTranslator(BaseMachineTranslator):
    display_name = "Google Translate"

    def translate(self, source_locale, target_locale, strings):
        client = translate.TranslationServiceClient()
        project_id = self.options["PROJECT_ID"]
        location = self.options.get("LOCATION", "global")

        response = client.translate_text(
            request={
                "parent": f"projects/{project_id}/locations/{location}",
                "contents": [string.data for string in strings],
                "mime_type": "text/html",
                "source_language_code": source_locale.language_code,
                "target_language_code": target_locale.language_code,
            }
        )

        return {
            string: StringValue(translation.translated_text)
            for string, translation in zip(strings, response.translations)
        }

    def can_translate(self, source_locale, target_locale):
        return source_locale.language_code != target_locale.language_code



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class WagtailLocalesAppConfig(AppConfig):
    name = "wagtail_localize.locales"
    label = "wagtaillocales"
    verbose_name = _("Wagtail locales (Wagtail Localize version)")



from django.contrib.messages import get_messages
from django.test import TestCase, override_settings
from django.urls import reverse
from wagtail.core.models import Locale
from wagtail.tests.utils import WagtailTestUtils

from wagtail_localize.locales.components import LOCALE_COMPONENTS
from wagtail_localize.models import LocaleSynchronization


@override_settings(WAGTAIL_CONTENT_LANGUAGES=[("en", "English"), ("fr", "French")])
class TestLocaleIndexView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

    def get(self, params=None):
        return self.client.get(reverse("wagtaillocales:index"), params or {})

    def test_simple(self):
        response = self.get()
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtaillocales/index.html")


@override_settings(WAGTAIL_CONTENT_LANGUAGES=[("en", "English"), ("fr", "French")])
class TestLocaleCreateView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()
        self.english = Locale.objects.get()

    def get(self, params=None):
        return self.client.get(reverse("wagtaillocales:add"), params or {})

    def post(self, post_data=None):
        return self.client.post(reverse("wagtaillocales:add"), post_data or {})

    def test_default_language(self):
        # we should have loaded with a single locale
        self.assertEqual(self.english.language_code, "en")
        self.assertEqual(self.english.get_display_name(), "English")

    def test_simple(self):
        response = self.get()
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtaillocales/create.html")

        self.assertEqual(
            response.context["form"].fields["language_code"].choices, [("fr", "French")]
        )

    def test_create(self):
        response = self.post(
            {
                "language_code": "fr",
                "component-wagtail_localize_localesynchronization-enabled": "on",
                "component-wagtail_localize_localesynchronization-sync_from": self.english.id,
            }
        )

        # Should redirect back to index
        self.assertRedirects(response, reverse("wagtaillocales:index"))

        # Check that the locale was created
        self.assertTrue(Locale.objects.filter(language_code="fr").exists())

        # Check the sync_from was set
        self.assertTrue(
            LocaleSynchronization.objects.filter(
                locale__language_code="fr", sync_from__language_code="en"
            ).exists()
        )

    def test_duplicate_not_allowed(self):
        response = self.post(
            {
                "language_code": "en",
                "component-wagtail_localize_localesynchronization-enabled": "on",
                "component-wagtail_localize_localesynchronization-sync_from": self.english.id,
            }
        )

        # Should return the form with errors
        self.assertEqual(response.status_code, 200)
        self.assertFormError(
            response,
            "form",
            "language_code",
            ["Select a valid choice. en is not one of the available choices."],
        )

    def test_language_code_must_be_in_settings(self):
        response = self.post(
            {
                "language_code": "ja",
                "component-wagtail_localize_localesynchronization-enabled": "on",
                "component-wagtail_localize_localesynchronization-sync_from": self.english.id,
            }
        )

        # Should return the form with errors
        self.assertEqual(response.status_code, 200)
        self.assertFormError(
            response,
            "form",
            "language_code",
            ["Select a valid choice. ja is not one of the available choices."],
        )

    def test_sync_from_required_when_enabled(self):
        response = self.post(
            {
                "language_code": "fr",
                "component-wagtail_localize_localesynchronization-enabled": "on",
                "component-wagtail_localize_localesynchronization-sync_from": "",
            }
        )

        # Should return the form with errors
        self.assertEqual(response.status_code, 200)
        self.assertFormError(
            response, "component_form", "sync_from", ["This field is required."]
        )

        # Check that the locale was not created
        self.assertFalse(Locale.objects.filter(language_code="fr").exists())

    def test_sync_from_not_required_when_disabled(self):
        response = self.post(
            {
                "language_code": "fr",
                "component-wagtail_localize_localesynchronization-enabled": "",
                "component-wagtail_localize_localesynchronization-sync_from": "",
            }
        )

        # Should redirect back to index
        self.assertRedirects(response, reverse("wagtaillocales:index"))

        # Check that the locale was created
        self.assertTrue(Locale.objects.filter(language_code="fr").exists())

        # Check the sync_from was not set
        self.assertFalse(LocaleSynchronization.objects.exists())

    def test_sync_from_required_when_component_required(self):
        LOCALE_COMPONENTS[0]["required"] = True
        try:
            response = self.post(
                {
                    "language_code": "fr",
                    "component-wagtail_localize_localesynchronization-enabled": "",
                    "component-wagtail_localize_localesynchronization-sync_from": "",
                }
            )
        finally:
            LOCALE_COMPONENTS[0]["required"] = False

        # Should return the form with errors
        self.assertEqual(response.status_code, 200)
        self.assertFormError(
            response, "component_form", "sync_from", ["This field is required."]
        )

        # Check that the locale was not created
        self.assertFalse(Locale.objects.filter(language_code="fr").exists())


@override_settings(WAGTAIL_CONTENT_LANGUAGES=[("en", "English"), ("fr", "French")])
class TestLocaleEditView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()
        self.english = Locale.objects.get()

    def get(self, params=None, locale=None):
        locale = locale or self.english
        return self.client.get(
            reverse("wagtaillocales:edit", args=[locale.id]), params or {}
        )

    def post(self, post_data=None, locale=None):
        post_data = post_data or {}
        locale = locale or self.english
        post_data.setdefault("language_code", locale.language_code)
        return self.client.post(
            reverse("wagtaillocales:edit", args=[locale.id]), post_data
        )

    def test_simple(self):
        response = self.get()
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtaillocales/edit.html")

        self.assertEqual(
            response.context["form"].fields["language_code"].choices,
            [
                (
                    "en",
                    "English",
                ),  # Note: Current value is displayed even though it's in use
                ("fr", "French"),
            ],
        )

    def test_invalid_language(self):
        invalid = Locale.objects.create(language_code="foo")

        response = self.get(locale=invalid)

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtaillocales/edit.html")

        self.assertEqual(
            response.context["form"].fields["language_code"].choices,
            [
                (
                    None,
                    "Select a new language",
                ),  # This is shown instead of the current value if invalid
                ("fr", "French"),
            ],
        )

    def test_edit(self):
        response = self.post(
            {
                "language_code": "fr",
                "component-wagtail_localize_localesynchronization-enabled": "on",
                "component-wagtail_localize_localesynchronization-sync_from": self.english.id,
            }
        )

        # Should redirect back to index
        self.assertRedirects(response, reverse("wagtaillocales:index"))

        # Check that the locale was edited
        self.english.refresh_from_db()
        self.assertEqual(self.english.language_code, "fr")

    def test_edit_duplicate_not_allowed(self):
        french = Locale.objects.create(language_code="fr")

        response = self.post(
            {
                "language_code": "en",
                "component-wagtail_localize_localesynchronization-enabled": "on",
                "component-wagtail_localize_localesynchronization-sync_from": self.english.id,
            },
            locale=french,
        )

        # Should return the form with errors
        self.assertEqual(response.status_code, 200)
        self.assertFormError(
            response,
            "form",
            "language_code",
            ["Select a valid choice. en is not one of the available choices."],
        )

    def test_edit_language_code_must_be_in_settings(self):
        response = self.post(
            {
                "language_code": "ja",
                "component-wagtail_localize_localesynchronization-enabled": "on",
                "component-wagtail_localize_localesynchronization-sync_from": self.english.id,
            }
        )

        # Should return the form with errors
        self.assertEqual(response.status_code, 200)
        self.assertFormError(
            response,
            "form",
            "language_code",
            ["Select a valid choice. ja is not one of the available choices."],
        )

    def test_sync_from_required_when_enabled(self):
        response = self.post(
            {
                "language_code": "fr",
                "component-wagtail_localize_localesynchronization-enabled": "on",
                "component-wagtail_localize_localesynchronization-sync_from": "",
            }
        )

        # Should return the form with errors
        self.assertEqual(response.status_code, 200)
        self.assertFormError(
            response, "component_form", "sync_from", ["This field is required."]
        )

    def test_sync_from_not_required_when_disabled(self):
        response = self.post(
            {
                "language_code": "fr",
                "component-wagtail_localize_localesynchronization-enabled": "",
                "component-wagtail_localize_localesynchronization-sync_from": "",
            }
        )

        # Should redirect back to index
        self.assertRedirects(response, reverse("wagtaillocales:index"))

        # Check that the locale was edited
        self.english.refresh_from_db()
        self.assertEqual(self.english.language_code, "fr")

    def test_sync_from_required_when_component_required(self):
        LOCALE_COMPONENTS[0]["required"] = True
        try:
            response = self.post(
                {
                    "language_code": "fr",
                    "component-wagtail_localize_localesynchronization-enabled": "",
                    "component-wagtail_localize_localesynchronization-sync_from": "",
                }
            )
        finally:
            LOCALE_COMPONENTS[0]["required"] = False

        # Should return the form with errors
        self.assertEqual(response.status_code, 200)
        self.assertFormError(
            response, "component_form", "sync_from", ["This field is required."]
        )

    def test_sync_from_cannot_be_the_same_as_locale(self):
        response = self.post(
            {
                "language_code": "en",
                "component-wagtail_localize_localesynchronization-enabled": "on",
                "component-wagtail_localize_localesynchronization-sync_from": self.english.id,
            }
        )

        # Should return the form with errors
        self.assertEqual(response.status_code, 200)
        self.assertFormError(
            response,
            "component_form",
            "sync_from",
            ["This locale cannot be synced into itself."],
        )


@override_settings(WAGTAIL_CONTENT_LANGUAGES=[("en", "English"), ("fr", "French")])
class TestLocaleDeleteView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()
        self.english = Locale.objects.get()

    def get(self, params=None, locale=None):
        locale = locale or self.english
        return self.client.get(
            reverse("wagtaillocales:delete", args=[locale.id]), params or {}
        )

    def post(self, post_data=None, locale=None):
        locale = locale or self.english
        return self.client.post(
            reverse("wagtaillocales:delete", args=[locale.id]), post_data or None
        )

    def test_simple(self):
        response = self.get()
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailadmin/generic/confirm_delete.html")

    def test_delete_locale(self):
        french = Locale.objects.create(language_code="fr")

        response = self.post(locale=french)

        # Should redirect back to index
        self.assertRedirects(response, reverse("wagtaillocales:index"))

        # Check that the locale was deleted
        self.assertFalse(Locale.objects.filter(language_code="fr").exists())

    def test_cannot_delete_locales_with_pages(self):
        response = self.post()

        self.assertEqual(response.status_code, 200)

        # Check error message
        messages = list(get_messages(response.wsgi_request))
        self.assertEqual(messages[0].level_tag, "error")
        self.assertEqual(
            messages[0].message,
            "This locale cannot be deleted because there are pages and/or other objects using it.\n\n\n\n\n",
        )

        # Check that the locale was not deleted
        self.assertTrue(Locale.objects.filter(language_code="en").exists())



# Generated by Django 3.0.6 on 2020-07-17 15:07

import django.db.models.deletion

from django.db import migrations, models
from django.db.migrations.recorder import MigrationRecorder
from wagtail import VERSION as WAGTAIL_VERSION


def get_revision_model():
    revision_model = "wagtailcore.PageRevision"
    if WAGTAIL_VERSION >= (4, 0, 0):
        try:
            if MigrationRecorder.Migration.objects.filter(
                app="wagtailcore", name="0070_rename_pagerevision_revision"
            ).exists():
                revision_model = "wagtailcore.Revision"
        except (django.db.utils.OperationalError, django.db.utils.ProgrammingError):
            # normally happens when running tests
            pass

    return revision_model


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("wagtailcore", "0057_page_locale_fields_notnull"),
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="String",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("data_hash", models.UUIDField()),
                ("data", models.TextField()),
                (
                    "locale",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="source_strings",
                        to="wagtailcore.Locale",
                    ),
                ),
            ],
            options={
                "unique_together": {("locale", "data_hash")},
            },
        ),
        migrations.CreateModel(
            name="Template",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("uuid", models.UUIDField(unique=True)),
                ("template", models.TextField()),
                ("template_format", models.CharField(max_length=100)),
                ("string_count", models.PositiveIntegerField()),
            ],
        ),
        migrations.CreateModel(
            name="TranslatableObject",
            fields=[
                (
                    "translation_key",
                    models.UUIDField(primary_key=True, serialize=False),
                ),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={
                "unique_together": {("content_type", "translation_key")},
            },
        ),
        migrations.CreateModel(
            name="TranslationSource",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("object_repr", models.TextField(max_length=200)),
                ("content_json", models.TextField()),
                ("created_at", models.DateTimeField()),
                (
                    "locale",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="wagtailcore.Locale",
                    ),
                ),
                (
                    "object",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="sources",
                        to="wagtail_localize.TranslatableObject",
                    ),
                ),
                (
                    "specific_content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="TranslationLog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                (
                    "locale",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translation_logs",
                        to="wagtailcore.Locale",
                    ),
                ),
                (
                    "page_revision",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=get_revision_model(),
                    ),
                ),
                (
                    "source",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translation_logs",
                        to="wagtail_localize.TranslationSource",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="TranslationContext",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("path_id", models.UUIDField()),
                ("path", models.TextField()),
                (
                    "object",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="wagtail_localize.TranslatableObject",
                    ),
                ),
            ],
            options={
                "unique_together": {("object", "path_id")},
            },
        ),
        migrations.CreateModel(
            name="TemplateSegment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("order", models.PositiveIntegerField()),
                (
                    "context",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="wagtail_localize.TranslationContext",
                    ),
                ),
                (
                    "source",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="wagtail_localize.TranslationSource",
                    ),
                ),
                (
                    "template",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="segments",
                        to="wagtail_localize.Template",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="StringSegment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("order", models.PositiveIntegerField()),
                ("attrs", models.TextField(blank=True)),
                (
                    "context",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="wagtail_localize.TranslationContext",
                    ),
                ),
                (
                    "source",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="wagtail_localize.TranslationSource",
                    ),
                ),
                (
                    "string",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="segments",
                        to="wagtail_localize.String",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="RelatedObjectSegment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("order", models.PositiveIntegerField()),
                (
                    "context",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="wagtail_localize.TranslationContext",
                    ),
                ),
                (
                    "object",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="references",
                        to="wagtail_localize.TranslatableObject",
                    ),
                ),
                (
                    "source",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="wagtail_localize.TranslationSource",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="StringTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("data", models.TextField()),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "context",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="translations",
                        to="wagtail_localize.TranslationContext",
                    ),
                ),
                (
                    "locale",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="string_translations",
                        to="wagtailcore.Locale",
                    ),
                ),
                (
                    "translation_of",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="wagtail_localize.String",
                    ),
                ),
            ],
            options={
                "unique_together": {("locale", "translation_of", "context")},
            },
        ),
    ]
